@@ -108,6 +108,47 @@ string StrStrip(string in_str) {
 
 }
 
+int XtoI(char x) {
+    if (isxdigit(x)) {
+        if (x <= '9')
+            return x - '0';
+        return toupper(x) - 'A' + 10;
+    }
+
+    return -1;
+}
+
+int Hex2UChar13(unsigned char *in_hex, unsigned char *in_chr) {
+    memset(in_chr, 0, sizeof(unsigned char) * WEPKEY_MAX);
+    int chrpos = 0;
+
+    for (unsigned int strpos = 0; strpos < WEPKEYSTR_MAX && chrpos < WEPKEY_MAX; strpos++) {
+        if (in_hex[strpos] == 0)
+            break;
+
+        if (in_hex[strpos] == ':')
+            strpos++;
+
+        // Assume we're going to eat the pair here
+        if (isxdigit(in_hex[strpos])) {
+            if (strpos > (WEPKEYSTR_MAX - 2))
+                return 0;
+
+            int d1, d2;
+            if ((d1 = XtoI(in_hex[strpos++])) == -1)
+                return 0;
+            if ((d2 = XtoI(in_hex[strpos])) == -1)
+                return 0;
+
+            in_chr[chrpos++] = (d1 * 16) + d2;
+        }
+
+    }
+
+    return(chrpos);
+}
+
+
 int ConfigFile::ParseConfig(const char *in_fname) {
     FILE *configf;
     char confline[8192];
@@ -146,7 +187,7 @@ int ConfigFile::ParseConfig(const char *in_fname) {
                 continue;
             }
 
-            config_map[StrLower(directive)] = value;
+            config_map[StrLower(directive)].push_back(value);
         }
     }
 
@@ -156,19 +197,32 @@ int ConfigFile::ParseConfig(const char *in_fname) {
 }
 
 string ConfigFile::FetchOpt(string in_key) {
+    map<string, vector<string> >::iterator cmitr = config_map.find(StrLower(in_key));
     // No such key
-    if (config_map.find(StrLower(in_key)) == config_map.end())
+    if (cmitr == config_map.end())
         return "";
 
-    // Get key
-    string val = config_map[StrLower(in_key)];
-    // Catch single-element stuff "DisableWEP" or whatever
-    if (val == "")
-        val = "true";
+    // Get a single key if we can
+    if (cmitr->second.size() == 0)
+        return "";
+
+    string val = cmitr->second[0];
 
     return val;
 }
 
+vector<string> ConfigFile::FetchOptVec(string in_key) {
+    // Empty vec to return
+    vector<string> eretvec;
+
+    map<string, vector<string> >::iterator cmitr = config_map.find(StrLower(in_key));
+    // No such key
+    if (cmitr == config_map.end())
+        return eretvec;
+
+    return cmitr->second;
+}
+
 // Expand a logfile into a full filename
 // Path/template from config
 // Logfile name to use
@@ -200,6 +254,18 @@ string ConfigFile::ExpandLogPath(string path, string logname, string type,
 
             logtemplate.insert(nl, datestr);
         }
+        else if (op == 't') {
+            time_t tnow;
+            struct tm *now;
+
+            tnow = time(0);
+            now = localtime(&tnow);
+
+            char timestr[12];
+            strftime(timestr, 12, "%H-%M-%S", now);
+
+            logtemplate.insert(nl, timestr);
+        }
         else if (op == 'l')
             logtemplate.insert(nl, type.c_str());
         else if (op == 'i')
