@@ -30,6 +30,7 @@
 
 #include <string>
 #include <map>
+#include <vector>
 
 // Munge a string to characters safe for calling in a shell
 void MungeToShell(char *in_data, int max);
@@ -38,16 +39,20 @@ string MungeToShell(string in_data);
 string StrLower(string in_str);
 string StrStrip(string in_str);
 
+int XtoI(char x);
+int Hex2UChar13(unsigned char *in_hex, unsigned char *in_chr);
+
 class ConfigFile {
 public:
 
     int ParseConfig(const char *in_fname);
     string FetchOpt(string in_key);
+    vector<string> FetchOptVec(string in_key);
 
-    string ExpandLogPath(string path, string logname, string type, int start, int overwrite = 0);
+    static string ExpandLogPath(string path, string logname, string type, int start, int overwrite = 0);
 
 protected:
-    map<string, string> config_map;
+    map<string, vector<string> > config_map;
 };
 
 #endif
