@@ -22,7 +22,7 @@
  */
 #ifndef lint
 static const char rcsid[] =
-    "@(#) $Header: /home/dragorn/src/CVS/kismet/kismet-devel/libpcap-2002.07.02/optimize.c,v 1.1.1.1 2002/07/22 15:01:27 dragorn Exp $ (LBL)";
+    "@(#) $Header: /home/dragorn/cvs/kismet/kismet-devel/libpcap-2002.12.23/optimize.c,v 1.1 2002/12/23 18:06:51 dragorn Exp $ (LBL)";
 #endif
 
 #ifdef HAVE_CONFIG_H
@@ -1945,7 +1945,7 @@ convert_code_r(p)
 
 	/* generate offset[] for convenience  */
 	if (slen) {
-		offset = (struct slist **)calloc(sizeof(struct slist *), slen);
+		offset = (struct slist **)calloc(slen, sizeof(struct slist *));
 		if (!offset) {
 			bpf_error("not enough core");
 			/*NOTREACHED*/
