@@ -26,7 +26,7 @@
  */
 #ifndef lint
 static const char rcsid[] =
-    "@(#) $Header: /home/dragorn/src/CVS/kismet/kismet-devel/libpcap-2002.07.02/pcap-linux.c,v 1.1.1.1 2002/07/22 15:01:27 dragorn Exp $ (LBL)";
+    "@(#) $Header: /home/dragorn/cvs/kismet/kismet-devel/libpcap-2002.12.23/pcap-linux.c,v 1.1 2002/12/23 18:06:51 dragorn Exp $ (LBL)";
 #endif
 
 /*
@@ -156,6 +156,16 @@ typedef int		socklen_t;
 #define MSG_TRUNC	0x20
 #endif
 
+#ifndef SOL_PACKET
+/*
+ * This is being compiled on a system that lacks SOL_PACKET; define it
+ * with the value it has in the 2.2 and later kernels, so that we can
+ * set promiscuous mode in the good modern way rather than the old
+ * 2.0-kernel crappy way.
+ */
+#define SOL_PACKET	263
+#endif
+
 #define MAX_LINKHEADER_SIZE	256
 
 /*
@@ -169,8 +179,8 @@ typedef int		socklen_t;
  * Prototypes for internal functions
  */
 static void map_arphrd_to_dlt(pcap_t *, int, int);
-static int live_open_old(pcap_t *, char *, int, int, char *);
-static int live_open_new(pcap_t *, char *, int, int, char *);
+static int live_open_old(pcap_t *, const char *, int, int, char *);
+static int live_open_new(pcap_t *, const char *, int, int, char *);
 static int pcap_read_packet(pcap_t *, pcap_handler, u_char *);
 
 /*
@@ -209,7 +219,8 @@ static struct sock_fprog	total_fcode
  *  See also pcap(3).
  */
 pcap_t *
-pcap_open_live(char *device, int snaplen, int promisc, int to_ms, char *ebuf)
+pcap_open_live(const char *device, int snaplen, int promisc, int to_ms,
+    char *ebuf)
 {
 	pcap_t		*handle;
 	int		mtu;
@@ -726,6 +737,20 @@ pcap_stats(pcap_t *handle, struct pcap_stat *stats)
 	return 0;
 }
 
+/*
+ * Description string for the "any" device.
+ */
+static const char any_descr[] = "Pseudo-device that captures on all interfaces";
+
+int
+pcap_platform_finddevs(pcap_if_t **alldevsp, char *errbuf)
+{
+	if (pcap_add_if(alldevsp, "any", 0, any_descr, errbuf) < 0)
+		return (-1);
+
+	return (0);
+}
+
 /*
  *  Attach the given BPF code to the packet capture device.
  */
@@ -1072,6 +1097,30 @@ static void map_arphrd_to_dlt(pcap_t *handle, int arptype, int cooked_ok)
 		handle->linktype = DLT_LTALK;
 		break;
 
+#ifndef ARPHRD_FCPP
+#define ARPHRD_FCPP	784
+#endif
+	case ARPHRD_FCPP:
+#ifndef ARPHRD_FCAL
+#define ARPHRD_FCAL	785
+#endif
+	case ARPHRD_FCAL:
+#ifndef ARPHRD_FCPL
+#define ARPHRD_FCPL	786
+#endif
+	case ARPHRD_FCPL:
+#ifndef ARPHRD_FCFABRIC
+#define ARPHRD_FCFABRIC	787
+#endif
+	case ARPHRD_FCFABRIC:
+		/*
+		 * We assume that those all mean RFC 2625 IP-over-
+		 * Fibre Channel, with the RFC 2625 header at
+		 * the beginning of the packet.
+		 */
+		handle->linktype = DLT_IP_OVER_FC;
+		break;
+
 	default:
 		handle->linktype = -1;
 		break;
@@ -1086,7 +1135,7 @@ static void map_arphrd_to_dlt(pcap_t *handle, int arptype, int cooked_ok)
  *  FIXME: 0 uses to mean success (Sebastian)
  */
 static int
-live_open_new(pcap_t *handle, char *device, int promisc,
+live_open_new(pcap_t *handle, const char *device, int promisc,
 	      int to_ms, char *ebuf)
 {
 #ifdef HAVE_PF_PACKET_SOCKETS
@@ -1224,7 +1273,6 @@ live_open_new(pcap_t *handle, char *device, int promisc,
 
 		/* Select promiscuous mode on/off */
 
-#ifdef SOL_PACKET
 		/*
 		 * Hmm, how can we set promiscuous mode on all interfaces?
 		 * I am not sure if that is possible at all.
@@ -1243,7 +1291,6 @@ live_open_new(pcap_t *handle, char *device, int promisc,
 				break;
 			}
 		}
-#endif
 
 		/* Save the socket FD in the pcap structure */
 
@@ -1442,7 +1489,7 @@ void	pcap_close_linux( pcap_t *handle )
  *  FIXME: 0 uses to mean success (Sebastian)
  */
 static int
-live_open_old(pcap_t *handle, char *device, int promisc,
+live_open_old(pcap_t *handle, const char *device, int promisc,
 	      int to_ms, char *ebuf)
 {
 	int		sock_fd = -1, arptype;
@@ -1888,3 +1935,9 @@ reset_kernel_filter(pcap_t *handle)
 				   &dummy, sizeof(dummy));
 }
 #endif
+
+int
+pcap_set_datalink_platform(pcap_t *p, int dlt)
+{
+	return (0);
+}
