@@ -30,7 +30,7 @@
 
 #ifndef lint
 static const char rcsid[] =
-    "@(#) $Header: /home/dragorn/src/CVS/kismet/kismet-devel/libpcap-2002.07.02/savefile.c,v 1.1.1.1 2002/07/22 15:01:27 dragorn Exp $ (LBL)";
+    "@(#) $Header: /home/dragorn/cvs/kismet/kismet-devel/libpcap-2002.12.23/savefile.c,v 1.1 2002/12/23 18:06:51 dragorn Exp $ (LBL)";
 #endif
 
 #ifdef HAVE_CONFIG_H
@@ -154,6 +154,7 @@ static const char rcsid[] =
 #define LINKTYPE_C_HDLC		104		/* Cisco HDLC */
 #define LINKTYPE_IEEE802_11	105		/* IEEE 802.11 (wireless) */
 #define LINKTYPE_ATM_CLIP	106		/* Linux Classical IP over ATM */
+#define LINKTYPE_FRELAY		107		/* Frame Relay */
 #define LINKTYPE_LOOP		108		/* OpenBSD loopback */
 
 #define LINKTYPE_LINUX_SLL	113		/* Linux cooked socket capture */
@@ -163,11 +164,14 @@ static const char rcsid[] =
 #define LINKTYPE_CISCO_IOS	118		/* For Cisco-internal use */
 #define LINKTYPE_PRISM_HEADER	119		/* 802.11+Prism II monitor mode */
 #define LINKTYPE_AIRONET_HEADER	120		/* FreeBSD Aironet driver stuff */
+#define LINKTYPE_IP_OVER_FC	122		/* RFC 2625 IP-over-Fibre Channel */
+#define LINKTYPE_SUNATM		123		/* Solaris+SunATM */
+
+#define LINKTYPE_IEEE802_11_RADIO 127		/* 802.11 plus WLAN header */
 
 /*
  * These types are reserved for future use.
  */
-#define LINKTYPE_FRELAY		107		/* Frame Relay */
 #define LINKTYPE_ENC		109		/* OpenBSD IPSEC enc */
 #define LINKTYPE_LANE8023	110		/* ATM LANE + 802.3 */
 #define LINKTYPE_HIPPI		111		/* NetBSD HIPPI */
@@ -175,8 +179,9 @@ static const char rcsid[] =
 #define LINKTYPE_IPFILTER	116		/* IP Filter capture files */
 #define LINKTYPE_PFLOG		117		/* OpenBSD DLT_PFLOG */
 #define LINKTYPE_HHDLC		121		/* Siemens HiPath HDLC */
-#define LINKTYPE_IP_OVER_FC	122		/* RFC 2625 IP-over-Fibre Channel */
-#define LINKTYPE_SUNATM		123		/* Solaris+SunATM */
+#define LINKTYPE_RIO		123		/* RapidIO */
+#define LINKTYPE_PCI_EXP	124		/* PCI Express */
+#define LINKTYPE_AURORA		125		/* Xilinx Aurora link layer */
 
 static struct linktype_map {
 	int	dlt;
@@ -268,6 +273,18 @@ static struct linktype_map {
 	/* Solaris+SunATM */
 	{ DLT_SUNATM,		LINKTYPE_SUNATM },
 
+	/* RapidIO */
+	{ DLT_RIO,		LINKTYPE_RIO },
+
+	/* PCI Express */
+	{ DLT_PCI_EXP,		LINKTYPE_PCI_EXP },
+
+	/* Xilinx Aurora link layer */
+	{ DLT_AURORA,		LINKTYPE_AURORA },
+
+	/* 802.11 plus WLAN header */
+	{ DLT_IEEE802_11_RADIO,	LINKTYPE_IEEE802_11_RADIO },
+
 	/*
 	 * Any platform that defines additional DLT_* codes should:
 	 *
@@ -375,12 +392,20 @@ pcap_open_offline(const char *fname, char *errbuf)
 	/*
 	 * Set this field so we don't close stdin in pcap_close!
 	 */
+#ifndef WIN32
 	p->fd = -1;
+#else
+	p->adapter = NULL;
+#endif
 
 	if (fname[0] == '-' && fname[1] == '\0')
 		fp = stdin;
 	else {
+#ifndef WIN32
 		fp = fopen(fname, "r");
+#else
+		fp = fopen(fname, "rb");
+#endif
 		if (fp == NULL) {
 			snprintf(errbuf, PCAP_ERRBUF_SIZE, "%s: %s", fname,
 			    pcap_strerror(errno));
@@ -421,7 +446,12 @@ pcap_open_offline(const char *fname, char *errbuf)
 	p->snapshot = hdr.snaplen;
 	p->linktype = linktype_to_dlt(hdr.linktype);
 	p->sf.rfile = fp;
+#ifndef WIN32
 	p->bufsize = hdr.snaplen;
+#else
+	/* Allocate the space for pcap_pkthdr as well. It will be used by pcap_read_ex */
+	p->bufsize = hdr.snaplen+sizeof(struct pcap_pkthdr);
+#endif
 
 	/* Align link header as required for proper data alignment */
 	/* XXX should handle all types */
@@ -458,6 +488,8 @@ pcap_open_offline(const char *fname, char *errbuf)
 
 	return (p);
  bad:
+	if(fp)
+		fclose(fp);
 	free(p);
 	return (NULL);
 }
@@ -467,7 +499,11 @@ pcap_open_offline(const char *fname, char *errbuf)
  * and the contents in buf.  Return 0 on success, SFERR_EOF if there were
  * no more packets, and SFERR_TRUNC if a partial packet was encountered.
  */
+#ifdef WIN32
+int
+#else
 static int
+#endif
 sf_next_packet(pcap_t *p, struct pcap_pkthdr *hdr, u_char *buf, int buflen)
 {
 	struct pcap_sf_patched_pkthdr sf_hdr;
@@ -633,10 +669,18 @@ pcap_dump_open(pcap_t *p, const char *fname)
 		return (NULL);
 	}
 
-	if (fname[0] == '-' && fname[1] == '\0')
+	if (fname[0] == '-' && fname[1] == '\0') {
 		f = stdout;
-	else {
+#ifdef WIN32
+		_setmode(_fileno(f), _O_BINARY);
+#endif
+	} else {
+#ifndef WIN32
 		f = fopen(fname, "w");
+#else
+		f = fopen(fname, "wb");
+		setbuf(f, NULL);	/* XXX - why? */
+#endif
 		if (f == NULL) {
 			snprintf(p->errbuf, PCAP_ERRBUF_SIZE, "%s: %s",
 			    fname, pcap_strerror(errno));
@@ -647,6 +691,16 @@ pcap_dump_open(pcap_t *p, const char *fname)
 	return ((pcap_dumper_t *)f);
 }
 
+int
+pcap_dump_flush(pcap_dumper_t *p)
+{
+
+	if (fflush((FILE *)p) == EOF)
+		return (-1);
+	else
+		return (0);
+}
+
 void
 pcap_dump_close(pcap_dumper_t *p)
 {
