@@ -39,6 +39,7 @@
 #endif
 #include <string>
 #include <vector>
+#include <map>
 
 // These won't include right if they're not in the C namespace
 extern "C" {
@@ -66,17 +67,17 @@ extern "C" {
 #define BEACON_INFO_LEN 128
 
 // Cribbed from ethereal, pointer to host endian swap
-#define kptoh16(p) (uint16_t) ((uint16_t) * ((uint8_t *)(p) + 1) << 8 | \
-                               (uint16_t) * ((uint8_t *)(p) + 0) << 0)
+#define kptoh16(p) (uint16_t) ((uint16_t) * ((uint8_t *)(p) + 0) << 8 | \
+                               (uint16_t) * ((uint8_t *)(p) + 1) << 0)
 
-#define kptoh24(p) (uint16_t) ((uint16_t) * ((uint8_t *)(p) + 2) << 16 | \
-                               (uint16_t) * ((uint8_t *)(p) + 1) << 8 | \
-                               (uint16_t) * ((uint8_t *)(p) + 0) << 0)
+#define kptoh24(p) (uint32_t) ((uint32_t) * ((uint8_t *)(p) + 0) << 16 | \
+                               (uint32_t) * ((uint8_t *)(p) + 1) << 8 | \
+                               (uint32_t) * ((uint8_t *)(p) + 2) << 0)
 
-#define kptoh32(p) (uint16_t) ((uint16_t) * ((uint8_t *)(p) + 3) << 24 | \
-                               (uint16_t) * ((uint8_t *)(p) + 2) << 16 | \
-                               (uint16_t) * ((uint8_t *)(p) + 1) << 8 | \
-                               (uint16_t) * ((uint8_t *)(p) + 0) << 0)
+#define kptoh32(p) (uint32_t) ((uint32_t) * ((uint8_t *)(p) + 0) << 24 | \
+                               (uint32_t) * ((uint8_t *)(p) + 1) << 16 | \
+                               (uint32_t) * ((uint8_t *)(p) + 2) << 8 | \
+                               (uint32_t) * ((uint8_t *)(p) + 3) << 0)
 
 // Inline converters
 #ifdef WORDS_BIGENDIAN
@@ -105,95 +106,98 @@ typedef struct packet_parm {
 typedef struct {
     unsigned int len;		// The amount of data we've actually got
     unsigned int caplen;	// The amount of data originally captured
-    struct timeval ts;
-    int quality;
-    int signal;
-    int noise;
-} pkthdr;
+    struct timeval ts;          // Capture timestamp
+    int quality;                // Signal quality
+    int signal;                 // Signal strength
+    int noise;                  // Noise level
+    int error;                  // Capture source told us this was a bad packet
+    uint8_t *data;              // Raw packet data
+    uint8_t *moddata;           // Modified packet data
+} kis_packet;
 
 #ifdef WORDS_BIGENDIAN
 // Byte ordering for bigendian systems.  Bitwise strcts are so funky.
 typedef struct {
-    unsigned short subtype : 4;
-    unsigned short type : 2;
-    unsigned short version : 2;
+    unsigned short subtype : 4 __attribute__ ((packed));
+    unsigned short type : 2 __attribute__ ((packed));
+    unsigned short version : 2 __attribute__ ((packed));
 
-    unsigned short order : 1;
-    unsigned short wep : 1;
-    unsigned short more_data : 1;
-    unsigned short power_management : 1;
+    unsigned short order : 1 __attribute__ ((packed));
+    unsigned short wep : 1 __attribute__ ((packed));
+    unsigned short more_data : 1 __attribute__ ((packed));
+    unsigned short power_management : 1 __attribute__ ((packed));
 
-    unsigned short retry : 1;
-    unsigned short more_fragments : 1;
-    unsigned short from_ds : 1;
-    unsigned short to_ds : 1;
+    unsigned short retry : 1 __attribute__ ((packed));
+    unsigned short more_fragments : 1 __attribute__ ((packed));
+    unsigned short from_ds : 1 __attribute__ ((packed));
+    unsigned short to_ds : 1 __attribute__ ((packed));
 } frame_control;
 
 typedef struct {
-    unsigned short frag : 12;
-    unsigned short sequence : 4;
+    unsigned short frag : 12 __attribute__ ((packed));
+    unsigned short sequence : 4 __attribute__ ((packed));
 } wireless_fragseq;
 
 typedef struct {
     uint8_t timestamp[8];
 
     // This field must be converted to host-endian before being used
-    unsigned int beacon : 16;
+    unsigned int beacon : 16 __attribute__ ((packed));
 
-    unsigned short agility : 1;
-    unsigned short pbcc : 1;
-    unsigned short short_preamble : 1;
-    unsigned short wep : 1;
+    unsigned short agility : 1 __attribute__ ((packed));
+    unsigned short pbcc : 1 __attribute__ ((packed));
+    unsigned short short_preamble : 1 __attribute__ ((packed));
+    unsigned short wep : 1 __attribute__ ((packed));
 
-    unsigned short unused2 : 1;
-    unsigned short unused1 : 1;
-    unsigned short ibss : 1;
-    unsigned short ess : 1;
+    unsigned short unused2 : 1 __attribute__ ((packed));
+    unsigned short unused1 : 1 __attribute__ ((packed));
+    unsigned short ibss : 1 __attribute__ ((packed));
+    unsigned short ess : 1 __attribute__ ((packed));
 
-    unsigned int coordinator : 8;
+    unsigned int coordinator : 8 __attribute__ ((packed));
 
 } fixed_parameters;
 
 #else
 // And 802.11 packet frame header
 typedef struct {
-    unsigned short version : 2;
-    unsigned short type : 2;
-    unsigned short subtype : 4;
+    unsigned short version : 2 __attribute__ ((packed));
+    unsigned short type : 2 __attribute__ ((packed));
+    unsigned short subtype : 4 __attribute__ ((packed));
 
-    unsigned short to_ds : 1;
-    unsigned short from_ds : 1;
-    unsigned short more_fragments : 1;
-    unsigned short retry : 1;
+    unsigned short to_ds : 1 __attribute__ ((packed));
+    unsigned short from_ds : 1 __attribute__ ((packed));
+    unsigned short more_fragments : 1 __attribute__ ((packed));
+    unsigned short retry : 1 __attribute__ ((packed));
 
-    unsigned short power_management : 1;
-    unsigned short more_data : 1;
-    unsigned short wep : 1;
-    unsigned short order : 1;
+    unsigned short power_management : 1 __attribute__ ((packed));
+    unsigned short more_data : 1 __attribute__ ((packed));
+    unsigned short wep : 1 __attribute__ ((packed));
+    unsigned short order : 1 __attribute__ ((packed));
 } frame_control;
 
 typedef struct {
-    unsigned short frag : 4;
-    unsigned short sequence : 12;
+    unsigned short frag : 4 __attribute__ ((packed));
+    unsigned short sequence : 12 __attribute__ ((packed));
 } wireless_fragseq;
 
 typedef struct {
     uint8_t timestamp[8];
 
     // This field must be converted to host-endian before being used
-    unsigned int beacon : 16;
+    unsigned int beacon : 16 __attribute__ ((packed));
 
-    unsigned short ess : 1;
-    unsigned short ibss : 1;
-    unsigned short unused1 : 1;
-    unsigned short unused2 : 1;
+    unsigned short ess : 1 __attribute__ ((packed));
+    unsigned short ibss : 1 __attribute__ ((packed));
+    unsigned short unused1 : 1 __attribute__ ((packed));
+    unsigned short unused2 : 1 __attribute__ ((packed));
 
-    unsigned short wep : 1;
-    unsigned short short_preamble : 1;
-    unsigned short pbcc : 1;
-    unsigned short agility : 1;
+    unsigned short wep : 1 __attribute__ ((packed));
+    unsigned short short_preamble : 1 __attribute__ ((packed));
+    unsigned short pbcc : 1 __attribute__ ((packed));
+    unsigned short agility : 1 __attribute__ ((packed));
 
-    unsigned int coordinator : 8;
+    unsigned int coordinator : 8 __attribute__ ((packed));
 } fixed_parameters;
 
 
@@ -206,9 +210,11 @@ enum protocol_info_type {
     proto_netbios, proto_netbios_tcp,
     proto_ipx,
     proto_ipx_tcp,
-    proto_lor,
+    proto_turbocell,
     proto_netstumbler,
-    proto_lucenttest
+    proto_lucenttest,
+    proto_wellenreiter,
+    proto_gstsearch
 };
 
 enum protocol_netbios_type {
@@ -222,18 +228,18 @@ enum protocol_netbios_type {
 // This spews a tremendous amount of revealing information about the
 // internals of a network, if they have cisco equipment.
 typedef struct {
-    unsigned int : 8;
-    unsigned int : 8;
+    unsigned int : 8 __attribute__ ((packed));
+    unsigned int : 8 __attribute__ ((packed));
 
-    unsigned int : 8;
-    unsigned int : 1;
-    unsigned int level1 : 1;
-    unsigned int igmp_forward : 1;
-    unsigned int nlp : 1;
-    unsigned int level2_switching : 1;
-    unsigned int level2_sourceroute : 1;
-    unsigned int level2_transparent : 1;
-    unsigned int level3 : 1;
+    unsigned int : 8 __attribute__ ((packed));
+    unsigned int : 1 __attribute__ ((packed));
+    unsigned int level1 : 1 __attribute__ ((packed));
+    unsigned int igmp_forward : 1 __attribute__ ((packed));
+    unsigned int nlp : 1 __attribute__ ((packed));
+    unsigned int level2_switching : 1 __attribute__ ((packed));
+    unsigned int level2_sourceroute : 1 __attribute__ ((packed));
+    unsigned int level2_transparent : 1 __attribute__ ((packed));
+    unsigned int level3 : 1 __attribute__ ((packed));
 } cdp_capabilities;
 
 typedef struct {
@@ -246,19 +252,19 @@ typedef struct {
 } cdp_packet;
 
 typedef struct {
-    unsigned int : 8;
-    unsigned int type : 8;
-    unsigned int : 8;
-    unsigned int length : 8;
+    unsigned int : 8 __attribute__ ((packed));
+    unsigned int type : 8 __attribute__ ((packed));
+    unsigned int : 8 __attribute__ ((packed));
+    unsigned int length : 8 __attribute__ ((packed));
     char data;
 } cdp_element;
 
 typedef struct {
-    unsigned int type : 8;
-    unsigned int length : 8;
-    unsigned int proto : 8;
-    unsigned int : 8;
-    unsigned int proto_length : 8;
+    unsigned int type : 8 __attribute__ ((packed));
+    unsigned int length : 8 __attribute__ ((packed));
+    unsigned int proto : 8 __attribute__ ((packed));
+    unsigned int : 8 __attribute__ ((packed));
+    unsigned int proto_length : 8 __attribute__ ((packed));
     char addr;
 } cdp_proto_element;
 
@@ -286,8 +292,6 @@ typedef struct proto_info {
     // Extra versioning/details of the proto type
     int prototype_extra;
 
-
-
 };
 
 
@@ -298,16 +302,26 @@ typedef struct mac_addr {
     uint8_t mac[MAC_LEN];
     uint64_t longmac;
     uint8_t mask;
+    int error;
 
     void struc2long() {
         longmac = 0;
-        for (int x = 0; x < MAC_LEN; x++)
+        error = 0;
+        for (int x = 0; x < MAC_LEN; x++) {
+            if (mac[x] > 0xFF) {
+                longmac = 0;
+                error = 1;
+                break;
+            }
+
             longmac |= (uint64_t) mac[x] << ((MAC_LEN - x - 1) * 8);
+        }
     }
 
     mac_addr() {
         memset(mac, 0, MAC_LEN);
         longmac = 0;
+        error = 0;
         mask = 0;
     }
 
@@ -315,6 +329,7 @@ typedef struct mac_addr {
         for (int x = 0; x < MAC_LEN; x++)
             mac[x] = in[x];
         mask = 0;
+        error = 0;
         struc2long();
     }
 
@@ -326,6 +341,8 @@ typedef struct mac_addr {
                    &bs_in[0], &bs_in[1], &bs_in[2], &bs_in[3], &bs_in[4], &bs_in[5]) == 6) {
             for (int x = 0; x < MAC_LEN; x++)
                 mac[x] = bs_in[x];
+        } else {
+            error = 1;
         }
         mask = 0;
         struc2long();
@@ -360,6 +377,8 @@ typedef struct mac_addr {
                    &bs_in[0], &bs_in[1], &bs_in[2], &bs_in[3], &bs_in[4], &bs_in[5]) == 6) {
             for (int x = 0; x < MAC_LEN; x++)
                 mac[x] = bs_in[x];
+        } else {
+            error = 1;
         }
         struc2long();
         return *this;
@@ -401,23 +420,65 @@ typedef struct mac_addr {
 
 
 // packet conversion and extraction utilities
-// Packet types
-enum packet_info_type {
-    packet_unknown, packet_beacon, packet_probe_req, packet_data,
-    packet_ap_broadcast, packet_adhoc, packet_adhoc_data,
-    packet_noise, packet_probe_response, packet_reassociation,
-    packet_auth, packet_deauth, packet_disassociation
+// Packet types, these should correspond to the frame header types
+enum packet_type {
+    packet_noise = -2,  // We're too short or otherwise corrupted
+    packet_unknown = -1, // What are we?
+    packet_management = 0, // LLC management
+    packet_phy = 1, // Physical layer packets, most drivers can't provide these
+    packet_data = 2, // Data frames
+};
+
+// Subtypes are a little odd because we re-use values depending on the type
+enum packet_sub_type {
+    packet_sub_unknown = -1,
+    // Management subtypes
+    packet_sub_association_req = 0,
+    packet_sub_association_resp = 1,
+    packet_sub_reassociation_req = 2,
+    packet_sub_reassociation_resp = 3,
+    packet_sub_probe_req = 4,
+    packet_sub_probe_resp = 5,
+    packet_sub_beacon = 8,
+    packet_sub_atim = 9,
+    packet_sub_disassociation = 10,
+    packet_sub_authentication = 11,
+    packet_sub_deauthentication = 12,
+    // Phy subtypes
+    packet_sub_rts = 11,
+    packet_sub_cts = 12,
+    packet_sub_ack = 13,
+    packet_sub_cf_end = 14,
+    packet_sub_cf_end_ack = 15,
+    // Data subtypes
+    packet_sub_data = 0,
+    packet_sub_data_cf_ack = 1,
+    packet_sub_data_cf_poll = 2,
+    packet_sub_data_cf_ack_poll = 3,
+    packet_sub_data_null = 4,
+    packet_sub_cf_ack = 5,
+    packet_sub_cf_ack_poll = 6
 };
 
 // distribution directions
 enum distribution_type {
-    no_distribution, from_distribution, to_distribution, inter_distribution
+    no_distribution, from_distribution, to_distribution, inter_distribution, adhoc_distribution
+};
+
+// Turbocell modes
+enum turbocell_type {
+    turbocell_unknown,
+    turbocell_ispbase, // 0xA0
+    turbocell_pollbase, // 0x80
+    turbocell_nonpollbase, // 0x00
+    turbocell_base // 0x40
 };
 
 // Info about a packet
 typedef struct {
     // Packet info type
-    packet_info_type type;
+    packet_type type;
+    packet_sub_type subtype;
 
     // reason code for some management protocols
     int reason_code;
@@ -437,12 +498,14 @@ typedef struct {
     distribution_type distrib;
     // Is wep enabled?
     int wep;
-    // Is this an AP or a adhoc?
-    int ap;
+    // Was it flagged as ess? (ap)
+    int ess;
     // What channel?
     int channel;
     // Is this encrypted?
     int encrypted;
+    // Did we decode it?
+    int decoded;
     // Is it weak crypto?
     int interesting;
 
@@ -468,20 +531,36 @@ typedef struct {
 
     int duration;
 
+    int datasize;
+
+    // Turbocell tracking info
+    int turbocell_nid;
+    turbocell_type turbocell_mode;
+    int turbocell_sat;
+
 } packet_info;
 
+typedef struct {
+    int fragile;
+    mac_addr bssid;
+    unsigned char key[13];
+    unsigned int len;
+} wep_key_info;
+
 // ----------------------------------
 // String munger
 void MungeToPrintable(char *in_data, int max);
 
 // Info extraction functions
-int GetTagOffset(int init_offset, int tagnum, const pkthdr *header, const u_char *data);
-void GetPacketInfo(const pkthdr *header, const uint8_t *data,
-                   packet_parm *parm, packet_info *ret_packinfo);
-void GetProtoInfo(const packet_info *in_info, const pkthdr *header,
-                  const u_char *in_data, proto_info *ret_protoinfo);
+int GetTagOffset(int init_offset, int tagnum, kis_packet *packet,
+                 map<int, int> *tag_cache_map);
+void GetPacketInfo(kis_packet *packet, packet_parm *parm, packet_info *ret_packinfo,
+                   map<mac_addr, wep_key_info *> *bssid_wep_map, unsigned char *identity);
+void GetProtoInfo(kis_packet *packet, packet_info *in_info);
+void DecryptPacket(kis_packet *packet, packet_info *in_info, 
+                   map<mac_addr, wep_key_info *> *bssid_wep_map, unsigned char *identity);
 
-vector<string> GetPacketStrings(const packet_info *in_info, const pkthdr *header, const u_char *in_data);
+vector<string> GetPacketStrings(const packet_info *in_info, kis_packet *packet);
 
 #endif
 
