@@ -30,6 +30,7 @@ char *KismetHelpText[] = {
     "KISMET PANELS INTERFACE",
     "QUICK REFERENCE",
     "  Key  Action",
+    "   e   List Kismet servers",
     "   z   Toggle fullscreen zoom of network view",
     "   m   Toggle muting of sound and speech",
     "   t   Tag (or untag) selected network",
@@ -69,7 +70,8 @@ char *KismetHelpText[] = {
     "  P       Probe request - no associated connection yet",
     "  A       Access point - standard wireless network",
     "  H       Ad-hoc - point-to-point wireless network",
-    "  O       Lucent - Lucent Outdoor Router network",
+    "  T       Turbocell - Turbocell (aka Karlnet or Lucent Outdoor",
+    "           Router) network",
     "  G       Group - Group of wireless networks",
     "  D       Data - Data only network with no control packets.",
     "",
@@ -101,6 +103,7 @@ char *KismetHelpTextNarrow[] = {
     "KISMET PANELS INTERFACE",
     "KISMET NETWORK PANEL",
     "Key Action",
+    " e  List Kismet servers",
     " z  Toggle fullscreen net list",
     " m  Toggle muting",
     " t  Tag (or untag) selected",
@@ -262,9 +265,37 @@ char *KismetHelpPack[] = {
     "simple history of a larger number of recent packets while the ",
     "bottom 3 quarters displays detailed information about a smaller",
     "number of packets.",
-    "'B' - Beacon      ; 'r' - Probe request  ; 'e' - Encrypted data",
-    "'w' - Weak data   ; 'd' - Data           ; 'a' - Ad-hoc",
-    "'n' - Noise       ; 'R' - Probe Response ; 'A' - Reassociation",
+    "'N ' - Noise",
+    "'U ' - Unknown",
+    "'Mx' - Management frame",
+    "  'Ma' - Association request",
+    "  'MA' - Association response",
+    "  'Mr' - Reassociation request",
+    "  'MR' - Reassociation response",
+    "  'Mp' - Probe request",
+    "  'MP' - Probe response",
+    "  'MB' - Beacon",
+    "  'MM' - ATIM",
+    "  'MD' - Disassociation",
+    "  'Mt' - Authentication",
+    "  'MT' - Deauthentication",
+    "  'M?' - Unknown management frame",
+    "'Px' - Physcial frame",
+    "  'Pt' - Request to send",
+    "  'PT' - Clear to send",
+    "  'PA' - Data Ack",
+    "  'Pc' - CF End",
+    "  'PC' - CF End+Ack",
+    "  'P?' - Unknown phy frame",
+    "'Dx' - Data frame",
+    "  'DD' - Data frame",
+    "  'Dc' - Data+CF+Ack",
+    "  'Dp' - Data+CF+Poll",
+    "  'DP' - Data+CF+Ack+Poll",
+    "  'DN' - Data Null",
+    "  'Da' - CF Ack",
+    "  'DA' - CF Ack+Poll",
+    "  'D?' - Unknown data frame",
     " Key   Action",
     "  p    Pause scrolling",
     "  q    Close popup",
@@ -321,6 +352,28 @@ char *KismetClientHelpDetails[] = {
     NULL
 };
 
+char *KismetHelpServer[] = {
+    "KISMET SERVERS",
+    " Key   Action",
+    " Up    Scroll list up",
+    " Down  Scroll list down",
+    "  t    Tag (or untag) selected server",
+    "  p    Make selected server the primary source",
+    "  c    Connect to new server",
+    "  d    Disconnect from selected server",
+    "  r    Reconnect to selected server",
+    "  q    Close server list",
+    "",
+    "Kismet supports monitoring data from several servers simultaneously.",
+    "When connected to multiple servers, only servers which are tagged",
+    "are displayed.  The server flagged as the 'primary' server is used for",
+    "GPS and time data.  Packet and network counts, packet rates, and",
+    "statistics are calculated for all of the available servers.  Networks",
+    "detected by two servers are displayed twice.",
+    "Servers tagged for display are denoted by a '*'",
+    "The primary server is denoted by a 'P'",
+    NULL
+};
 
 PanelFront::PanelFront() {
     errstr[0] = '\0';
@@ -339,6 +392,7 @@ PanelFront::PanelFront() {
     muted = 0;
 
     // Push blanks into the RRD history vector
+    packet_history.reserve(60 * 5);
     for (unsigned int x = 0; x < (60 * 5); x++)
         packet_history.push_back(0);
 
@@ -349,6 +403,51 @@ PanelFront::PanelFront() {
     last_lat = last_lon = last_alt = last_spd = 0;
     last_fix = 0;
 
+    num_networks = num_packets = num_crypt = num_interesting = num_noise =
+        num_dropped = packet_rate = 0;
+
+    context = NULL;
+
+    tainted = 0;
+}
+
+PanelFront::~PanelFront() {
+    // Delete the dynamically allocated contexts
+    for (unsigned int x = 0; x < context_list.size(); x++)
+        delete context_list[x];
+}
+
+void PanelFront::AddClient(TcpClient *in_client) {
+    server_context *new_context = new server_context;
+    new_context->client = in_client;
+    context_list.push_back(new_context);
+    client_list.push_back(in_client);
+
+    new_context->tagged = 1;
+
+    if (context == NULL) {
+        client = in_client;
+        context = new_context;
+        context->primary = 1;
+    }
+
+    // Enable all the protocols we handle
+    client->EnableProtocol("GPS");
+    client->EnableProtocol("INFO");
+    client->EnableProtocol("NETWORK");
+    client->EnableProtocol("CLIENT");
+    client->EnableProtocol("REMOVE");
+    client->EnableProtocol("ALERT");
+    client->EnableProtocol("STATUS");
+}
+
+void PanelFront::FetchClients(vector<TcpClient *> *in_vec) {
+    in_vec->clear();
+    *in_vec = client_list;
+}
+
+TcpClient *PanelFront::FetchPrimaryClient() {
+    return client;
 }
 
 int PanelFront::InitDisplay(int in_decay, time_t in_start) {
@@ -534,6 +633,8 @@ PanelFront::main_columns PanelFront::Token2MainColumn(string in_token) {
         return mcol_noise;
     } else if (in_token == "clients") {
         return mcol_clients;
+    } else if (in_token == "size") {
+        return mcol_datasize;
     } else {
         return mcol_unknown;
     }
@@ -567,6 +668,8 @@ PanelFront::client_columns PanelFront::Token2ClientColumn(string in_token) {
         return ccol_quality;
     } else if (in_token == "noise") {
         return ccol_noise;
+    } else if (in_token == "size") {
+        return ccol_datasize;
     }
 
     return ccol_unknown;
@@ -610,6 +713,8 @@ void PanelFront::SetClientColumns(string in_columns) {
 int PanelFront::WriteStatus(string status) {
     stat_win->text.push_back(status);
 
+    tainted = 1;
+
     return 1;
 
     /*
@@ -676,6 +781,8 @@ int PanelFront::DrawDisplay() {
     update_panels();
     doupdate();
 
+    tainted = 0;
+
     return 1;
 }
 
@@ -869,6 +976,8 @@ int PanelFront::Poll() {
     if ((ch = wgetch(cur_window->win)) == ERR)
         return 0;
 
+    // fprintf(stderr, "Got: %d (%c)\n", ch, ch);
+
     // Catch the redraw event ^L
     if (ch == 12) {
         clearok(curscr, 1);
@@ -891,32 +1000,119 @@ int PanelFront::Poll() {
     return ret;
 }
 
+void PanelFront::UpdateContexts() {
+    lat = lon = alt = spd = 0;
+    fix = 0;
+    last_lat = last_lon = last_alt = last_spd = 0;
+    last_fix = 0;
+
+    quality = power = noise = 0;
+
+    num_networks = 0;
+    num_packets = 0;
+    num_crypt = 0;
+    num_interesting = 0;
+    num_noise = 0;
+    num_dropped = 0;
+    packet_rate = 0;
+
+    int aggrate = 0; int aggadjrate = 0;
+
+    for (unsigned int x = 0; x < context_list.size(); x++) {
+        server_context *con = context_list[x];
+
+        if (con->client == NULL)
+            continue;
+
+        // Update GPS
+        con->last_lat = con->lat;
+        con->last_lon = con->lon;
+        con->last_spd = con->spd;
+        con->last_alt = con->alt;
+        con->last_fix = con->fix;
+        con->client->FetchLoc(&con->lat, &con->lon, &con->alt, &con->spd, &con->fix);
+
+        // Update quality
+        con->quality = con->client->FetchQuality();
+        con->power = con->client->FetchPower();
+        con->noise = con->client->FetchNoise();
+
+        // Update time
+        con->server_time = con->client->FetchTime();
+
+        if (con->primary == 1) {
+            // Bring the primary contexts info into our class settings
+            client = con->client;
+
+            lat = con->lat;
+            lon = con->lon;
+            alt = con->alt;
+            spd = con->spd;
+            fix = con->fix;
+
+            last_lat = con->last_lat;
+            last_lon = con->last_lon;
+            last_alt = con->last_alt;
+            last_spd = con->last_spd;
+            last_fix = con->last_fix;
+
+            quality = con->quality;
+            power = con->power;
+            noise = con->noise;
+
+            server_time = con->server_time;
+        }
+
+
+        // Update packet rate
+        int rate = con->client->FetchNumPackets() - con->client->FetchNumDropped();
+        int adjrate = con->client->FetchPacketRate();
+
+        if (adjrate > con->max_packet_rate)
+            con->max_packet_rate = adjrate;
+        con->packet_history.push_back(rate);
+        if (con->packet_history.size() > (60 * 5))
+            con->packet_history.erase(con->packet_history.begin());
+
+        // Update other info
+        con->num_networks = con->client->FetchNumNetworks();
+        con->num_packets = con->client->FetchNumPackets();
+        con->num_crypt = con->client->FetchNumCrypt();
+        con->num_interesting = con->client->FetchNumInteresting();
+        con->num_noise = con->client->FetchNumNoise();
+        con->num_dropped = con->client->FetchNumDropped();
+        con->packet_rate = con->client->FetchPacketRate();
+
+        if (con->tagged) {
+            // combine tagged info
+            aggrate += rate;
+            aggadjrate += adjrate;
+
+            num_networks += con->num_networks;
+            num_packets += con->num_packets;
+            num_crypt += con->num_crypt;
+            num_interesting += con->num_interesting;
+            num_noise += con->num_noise;
+            num_dropped += con->num_dropped;
+        }
+
+    }
+
+    packet_rate = aggadjrate;
+    if (aggadjrate > max_packet_rate)
+        max_packet_rate = aggadjrate;
+    packet_history.push_back(aggrate);
+    if (packet_history.size() > (60 * 5))
+        packet_history.erase(packet_history.begin());
+
+}
+
 int PanelFront::Tick() {
     // We should be getting a 1-second tick - secondary to a draw event, because
     // we can cause our own draw events which wouldn't necessarily be a good thing
 
-    last_lat = lat; last_lon = lon;
-    last_spd = spd; last_alt = alt;
-    last_fix = fix;
-
-    client->FetchLoc(&lat, &lon, &alt, &spd, &fix);
-
-
-    // Pull our packet count, store it, and bounce if we're
-    // holding more than 5 minutes worth.
-
-    int rate = client->FetchNumPackets() - client->FetchNumDropped();
-
-    int adjrate = client->FetchPacketRate();
-    if (adjrate > max_packet_rate)
-        max_packet_rate = adjrate;
-
-
-    packet_history.push_back(rate);
-
-    if (packet_history.size() > (60 * 5))
-        packet_history.erase(packet_history.begin());
-
+    // Update all the contexts
+    UpdateContexts();
 
     // Now fetch the APM data (if so desired)
     if (monitor_bat) {
@@ -972,27 +1168,27 @@ int PanelFront::Tick() {
         struct dirent* this_battery;
         FILE *acpi;
         char battery_state[PATH_MAX];
-
         int rate = 1, remain = 0, current = 0;
         static int total_remain = 0, total_cap = 0;
         int batno = 0;
         const int info_res = 5;
         static int info_timer = 0;
         batteries = opendir("/proc/acpi/battery");
-        if (!bat_available || ((info_timer % info_res) == 0)) {
+        if (!bat_available || ((info_timer % info_res) == 0))
+        {
             bat_ac = 0;
             bat_percentage = 0;
             bat_time = 0;
             bat_charging = 0;
             total_remain = total_cap = 0;
         }
-        while (((info_timer % info_res) == 0) && ((this_battery = readdir(batteries)) != NULL)) {
+        while (((info_timer % info_res) == 0) && ((this_battery = readdir(batteries)) != NULL))
+        {
             if (this_battery->d_name[0] == '.')
                 continue;
             snprintf(battery_state, sizeof(battery_state), "/proc/acpi/battery/%s/state", this_battery->d_name);
             if ((acpi = fopen(battery_state, "r")) == NULL)
                 continue;
-
             while (fgets(buf, 128, acpi))
             {
                 if (strncmp(buf, "present:", 8 ) == 0)
@@ -1009,10 +1205,12 @@ int PanelFront::Tick() {
                 }
                 else if (strncmp(buf, "present rate:", 13) == 0)
                     rate = atoi(buf + 25);
-                else if (strncmp(buf, "remaining capacity:", 19) == 0) {
+                else if (strncmp(buf, "remaining capacity:", 19) == 0)
+                {
                     remain = atoi(buf + 25);
                     total_remain += remain;
-                } else if (strncmp(buf, "present voltage:", 17) == 0)
+                }
+                else if (strncmp(buf, "present voltage:", 17) == 0)
                     current = atoi(buf + 25);
             }
             total_cap += bat_full_capacity[batno];
@@ -1049,7 +1247,8 @@ void PanelFront::AddPrefs(map<string, string> in_prefs) {
     int batno = 0;
     bat_available = 0;
     batteries = opendir("/proc/acpi/battery");
-    while ((this_battery = readdir(batteries)) != NULL) {
+    while ((this_battery = readdir(batteries)) != NULL)
+    {
         // Skip . and ..
         if (this_battery->d_name[0] == '.')
             continue;
@@ -1067,7 +1266,6 @@ void PanelFront::AddPrefs(map<string, string> in_prefs) {
     }
     closedir(batteries);
 #endif
-
     if (prefs["apm"] == "true")
         monitor_bat = 1;
     else
@@ -1128,5 +1326,4 @@ PanelFront::color_pair PanelFront::ColorParse(string in_color) {
     return ret;
 }
 
-
 #endif
