@@ -27,7 +27,6 @@
 
 int PanelFront::MainInput(void *in_window, int in_chr) {
     kis_window *kwin = (kis_window *) in_window;
-    char sendbuf[1024];
 
     switch (in_chr) {
     case 'Q':
@@ -62,6 +61,21 @@ int PanelFront::MainInput(void *in_window, int in_chr) {
             WriteStatus("Cannot scroll in autofit sort mode.");
         }
         break;
+    case KEY_PPAGE:
+        if (sortby != sort_auto) {
+            kwin->selected = -1; // We want to start a page in reverse
+        } else {
+            WriteStatus("Cannot scroll in autofit sort mode.");
+        }
+
+        break;
+    case KEY_NPAGE:
+        if (sortby != sort_auto) {
+            kwin->selected = -2; // We want to start a page forward
+        } else {
+            WriteStatus("Cannot scroll in autofit sort mode.");
+        }
+        break;
     case KEY_RIGHT:
     case '+':
         if (sortby != sort_auto && last_displayed.size() > 0) {
@@ -125,7 +139,8 @@ int PanelFront::MainInput(void *in_window, int in_chr) {
     case 'U':
         if (sortby != sort_auto && last_displayed.size() > 0) {
             if (last_displayed[kwin->selected] != NULL)
-                DestroyGroup(last_displayed[kwin->selected]);
+                if (last_displayed[kwin->selected]->type == group_bundle)
+                    DestroyGroup(last_displayed[kwin->selected]);
         } else {
             WriteStatus("Cannot ungroup in autofit sort mode.");
         }
@@ -160,8 +175,7 @@ int PanelFront::MainInput(void *in_window, int in_chr) {
         break;
     case 'd':
     case 'D':
-        snprintf(sendbuf, 1024, "!%u strings \n", (unsigned int) time(0));
-        client->Send(sendbuf);
+        client->EnableProtocol("STRING");
         WriteStatus("Requesting strings from the server");
 
         SpawnWindow("Data Strings Dump", &PanelFront::DumpPrinter, &PanelFront::DumpInput);
@@ -180,8 +194,7 @@ int PanelFront::MainInput(void *in_window, int in_chr) {
         break;
     case 'p':
     case 'P':
-        snprintf(sendbuf, 1024, "!%u packtypes \n", (unsigned int) time(0));
-        client->Send(sendbuf);
+        client->EnableProtocol("PACKET");
         WriteStatus("Requesting packet types from the server");
 
         SpawnWindow("Packet Types", &PanelFront::PackPrinter, &PanelFront::PackInput);
@@ -199,8 +212,13 @@ int PanelFront::MainInput(void *in_window, int in_chr) {
     case 'M':
         MuteToggle();
         break;
+    case 'e':
+    case 'E':
+        SpawnWindow("Kismet Servers", &PanelFront::ServersPrinter, &PanelFront::ServersInput, 10, 62);
+        break;
     }
 
+
     return 1;
 }
 
@@ -284,7 +302,6 @@ int PanelFront::SortInput(void *in_window, int in_chr) {
 
 int PanelFront::PackInput(void *in_window, int in_chr) {
     kis_window *kwin = (kis_window *) in_window;
-    char sendbuf[1024];
 
     switch(in_chr) {
     case 'h':
@@ -303,8 +320,7 @@ int PanelFront::PackInput(void *in_window, int in_chr) {
     case 'X':
     case 'q':
     case 'Q':
-        snprintf(sendbuf, 1024, "!%u nopacktypes\n", (unsigned int) time(0));
-        client->Send(sendbuf);
+        client->RemoveProtocol("PACKET");
         return 0;
         break;
     default:
@@ -317,7 +333,6 @@ int PanelFront::PackInput(void *in_window, int in_chr) {
 
 int PanelFront::DumpInput(void *in_window, int in_chr) {
     kis_window *kwin = (kis_window *) in_window;
-    char sendbuf[1024];
 
     switch(in_chr) {
     case 'm':
@@ -349,8 +364,7 @@ int PanelFront::DumpInput(void *in_window, int in_chr) {
     case 'X':
     case 'q':
     case 'Q':
-        snprintf(sendbuf, 1024, "!%u nostrings \n", (unsigned int) time(0));
-        client->Send(sendbuf);
+        client->RemoveProtocol("STRING");
         return 0;
         break;
     default:
@@ -474,6 +488,14 @@ int PanelFront::TextInput(void *in_window, int in_chr) {
     kis_window *kwin = (kis_window *) in_window;
 
     switch (in_chr) {
+    case KEY_PPAGE:
+        if (kwin->start != 0)
+            kwin->start = max(0, kwin->start - kwin->max_display);
+        break;
+    case KEY_NPAGE:
+        if (kwin->end < (int) kwin->text.size() - 1 && kwin->end != 0)
+            kwin->start = min((int)kwin->text.size() - kwin->max_display, kwin->start + kwin->max_display);
+        break;
     case KEY_UP:
     case '-':
         if (kwin->start != 0) {
@@ -579,6 +601,30 @@ int PanelFront::MainClientInput(void *in_window, int in_chr) {
             WriteStatus("Cannot scroll clients in autofit sort mode.");
         }
         break;
+    case KEY_PPAGE:
+        if (client_sortby != client_sort_auto) {
+            if (kwin->selected == 0 && kwin->start != 0) {
+                kwin->start -= kwin->max_display;
+                kwin->start = max(kwin->start,0);
+            } else if (kwin->selected > 0) {
+                kwin->selected -= kwin->max_display;
+                kwin->selected = max(kwin->selected,0);
+            }
+        } else {
+            WriteStatus("Cannot scroll clients in autofit sort mode.");
+        }
+
+        break;
+    case KEY_NPAGE:
+        if (client_sortby != client_sort_auto) {
+            if (kwin->start + kwin->selected + kwin->max_display < last_client_draw_size)
+                kwin->start += kwin->max_display - 1;
+            else
+                kwin->selected = kwin->end - kwin->start;
+        } else {
+            WriteStatus("Cannot scroll clients in autofit sort mode.");
+        }
+        break;
     case 's':
     case 'S':
         SpawnWindow("Sort Clients", &PanelFront::SortClientPrinter,
@@ -713,5 +759,102 @@ int PanelFront::DetailsClientInput(void *in_window, int in_chr) {
     return 1;
 }
 
+int PanelFront::ServersInput(void *in_window, int in_chr) {
+    kis_window *kwin = (kis_window *) in_window;
+    char msg[1024];
+
+    switch (in_chr) {
+    case 'h':
+    case 'H':
+        SpawnHelp(KismetHelpServer);
+        break;
+    case KEY_UP:
+        if (kwin->selected > 0) 
+            kwin->selected--;
+        else if (kwin->selected == 0 && kwin->start > 0)
+            kwin->start--;
+        break;
+    case KEY_DOWN:
+        if ((kwin->selected + kwin->start) < kwin->end)
+            kwin->selected++;
+        else if (kwin->end < ((int) context_list.size() - 1))
+            kwin->start++;
+        break;
+    case 'c':
+    case 'C':
+        SpawnWindow("New Server", &PanelFront::ServerJoinPrinter, NULL, 3, 40);
+        break;
+    case 'd':
+    case 'D':
+        if ((kwin->start + kwin->selected) < (int) context_list.size()) {
+            server_context *con = context_list[kwin->start + kwin->selected];
+            if (con->client != NULL) {
+                snprintf(msg, 1024, "Disconnecting from %s:%d",
+                         con->client->FetchHost(), con->client->FetchPort());
+                WriteStatus(msg);
+                con->client->Disconnect();
+            }
+        }
+        break;
+    case 'r':
+    case 'R':
+        if ((kwin->start + kwin->selected) < (int) context_list.size()) {
+            server_context *con = context_list[kwin->start + kwin->selected];
+            if (con->client != NULL) {
+                snprintf(msg, 1024, "Reconnecting to %s:%d",
+                         con->client->FetchHost(), con->client->FetchPort());
+                WriteStatus(msg);
+                con->client->Connect(con->client->FetchPort(), con->client->FetchHost());
+            }
+        }
+        break;
+    case 't':
+    case 'T':
+        if ((kwin->start + kwin->selected) < (int) context_list.size()) {
+            server_context *con = context_list[kwin->start + kwin->selected];
+
+            if (con->primary)
+                return 1;
+
+            if (con->tagged) {
+                con->tagged = 0;
+
+                // Flush the rate graph
+                packet_history.clear();
+                for (unsigned int x = 0; x < (60 * 5); x++)
+                    packet_history.push_back(0);
+                // Flush all the group mappings so we can repopulate with just the
+                // groups we have tagged now
+                PurgeGroups();
+            } else {
+                con->tagged = 1;
+            }
+        }
+        break;
+    case 'p':
+    case 'P':
+        if ((kwin->start + kwin->selected) < (int) context_list.size()) {
+            server_context *con = context_list[kwin->start + kwin->selected];
+
+            if (con->primary)
+                return 1;
+
+            for (unsigned int x = 0; x < context_list.size(); x++)
+                if (context_list[x]->primary)
+                    context_list[x]->primary = 0;
+
+            con->primary = 1;
+            con->tagged = 1;
+            context = con;
+        }
+        break;
+    case 'q':
+    case 'Q':
+        return 0;
+        break;
+    }
+
+    return 1;
+}
 
 #endif
