@@ -29,6 +29,9 @@ TcpClient::TcpClient() {
     num_networks = num_packets = num_crypt = num_interesting =
         num_noise = num_dropped = packet_rate = 0;
 
+    old_num_networks = old_num_packets = old_num_crypt = old_num_interesting =
+        old_num_noise = old_num_dropped = 0;
+
     start_time = 0;
     major = minor = tiny = 0;
 
@@ -43,6 +46,8 @@ TcpClient::TcpClient() {
 
     last_new_network = 0;
 
+    servername[0] = '\0';
+
 }
 
 TcpClient::~TcpClient() {
@@ -52,6 +57,13 @@ TcpClient::~TcpClient() {
     sv_valid = 0;
 }
 
+void TcpClient::Disconnect() {
+    if (sv_valid)
+        close(client_fd);
+
+    sv_valid = 0;
+}
+
 int TcpClient::Connect(short int in_port, char *in_host) {
     // Copy the port to our local data
     port = in_port;
@@ -66,7 +78,8 @@ int TcpClient::Connect(short int in_port, char *in_host) {
     strncpy(hostname, in_host, MAXHOSTNAMELEN);
 
     // Set up our socket
-    bzero(&client_sock, sizeof(client_sock));
+    //bzero(&client_sock, sizeof(client_sock));
+    memset(&client_sock, 0, sizeof(client_sock));
     client_sock.sin_family = client_host->h_addrtype;
     memcpy((char *) &client_sock.sin_addr.s_addr, client_host->h_addr_list[0],
            client_host->h_length);
@@ -74,7 +87,7 @@ int TcpClient::Connect(short int in_port, char *in_host) {
 
     if ((client_fd = socket(AF_INET, SOCK_STREAM, 0)) < 0) {
         snprintf(errstr, 1024, "TcpClient socket() failed %d (%s)\n",
-                 errno, sys_errlist[errno]);
+                 errno, strerror(errno));
         return (-2);
     }
 
@@ -85,26 +98,26 @@ int TcpClient::Connect(short int in_port, char *in_host) {
 
     if (bind(client_fd, (struct sockaddr *) &local_sock, sizeof(local_sock)) < 0) {
         snprintf(errstr, 1024, "FATAL: TcpClient bind() failed %d (%s)\n",
-                 errno, sys_errlist[errno]);
+                 errno, strerror(errno));
         return (-3);
     }
 
     // Connect
     if (connect(client_fd, (struct sockaddr *) &client_sock, sizeof(client_sock)) < 0) {
         snprintf(errstr, 1024, "FATAL: TcpClient connect() failed %d (%s)\n",
-                 errno, sys_errlist[errno]);
+                 errno, strerror(errno));
         return (-4);
     }
 
     int save_mode = fcntl(client_fd, F_GETFL, 0);
     if (save_mode == -1) {
         snprintf(errstr, 1024, "FATAL:  TcpClient connect() failed fcntl get %d (%s)\n",
-                 errno, sys_errlist[errno]);
+                 errno, strerror(errno));
         return (-5);
     }
     if (fcntl(client_fd, F_SETFL, save_mode | O_NONBLOCK) < 0) {
         snprintf(errstr, 1024, "FATAL:  TcpClient connect() failed fcntl set %d (%s)\n",
-                 errno, sys_errlist[errno]);
+                 errno, strerror(errno));
         return (-6);
     }
 
@@ -112,6 +125,14 @@ int TcpClient::Connect(short int in_port, char *in_host) {
 
     clientf = fdopen(client_fd, "r");
 
+    // Turn on the protocols in our map, if we have any.  This is for reconnecting
+    // to a server and preserving the requested protocols
+    for (map<string, int>::iterator prot = protocol_map.begin();
+         prot != protocol_map.end(); ++prot) {
+        EnableProtocol(prot->first);
+    }
+
+
     return 1;
 }
 
@@ -128,13 +149,11 @@ int TcpClient::Poll() {
     int selected;
     fd_set read_set;
     fd_set write_set;
-    fd_set except_set;
 
     FD_ZERO(&read_set);
     FD_SET(client_fd, &read_set);
     FD_ZERO(&write_set);
     FD_SET(client_fd, &write_set);
-    FD_ZERO(&except_set);
 
     struct timeval tim;
 
@@ -142,23 +161,16 @@ int TcpClient::Poll() {
     tim.tv_usec = 0;
 
     // Enter the select loop
-    if ((selected = select(client_fd+1, &read_set, &write_set, &except_set, &tim)) < 0) {
+    if ((selected = select(client_fd+1, &read_set, &write_set, NULL, &tim)) < 0) {
         if (errno != EINTR) {
             snprintf(errstr, 1024, "TcpServer select() returned %d (%s)\n",
-                     errno, sys_errlist[errno]);
+                     errno, strerror(errno));
             sv_valid = 0;
             close(client_fd);
             return (-1);
         }
     }
 
-    if (FD_ISSET(client_fd, &except_set)) {
-        snprintf(errstr, 1024, "Exception on socket.\n");
-        sv_valid = 0;
-        close(client_fd);
-        return(-1);
-    }
-
     if (writebuf.length() > 0 && FD_ISSET(client_fd, &write_set)) {
         int res = write(client_fd, writebuf.c_str(), writebuf.length());
 
@@ -183,15 +195,9 @@ int TcpClient::Poll() {
 
     //    while (1) {
     if (fgets(data, 2048, clientf) == NULL) {
-        /*
-        if (errno != 0)
-            snprintf(errstr, 1024, "Errno %d %s ferror %d\n",
-            errno, sys_errlist[errno], ferror(clientf));
-            */
-
         if (errno != 0 && errno != EAGAIN) {
             snprintf(errstr, 1024, "Read error %d (%s), closing the connection.",
-                     errno, sys_errlist[errno]);
+                     errno, strerror(errno));
             sv_valid = 0;
             close(client_fd);
             return (-1);
@@ -251,8 +257,8 @@ int TcpClient::ParseData(char *in_data) {
         snprintf(errstr, 1024, "Server has terminated.\n");
         return -1;
     } else if (!strncmp(header, "*KISMET", 64)) {
-        if (sscanf(in_data+hdrlen, "%d.%d.%d %d",
-                   &major, &minor, &tiny, (int *) &start_time) < 1)
+        if (sscanf(in_data+hdrlen, "%d.%d.%d %d \001%32[^\001]\001 %24s",
+                   &major, &minor, &tiny, (int *) &start_time, servername, build) < 6)
             return 0;
     } else if (!strncmp(header, "*TIME", 64)) {
         if (sscanf(in_data+hdrlen, "%d", (int *) &serv_time) < 1)
@@ -273,21 +279,22 @@ int TcpClient::ParseData(char *in_data) {
             return 0;
 
         bssid = bssid_str;
+        int newnet = 0;
 
         if (net_map.find(bssid) != net_map.end()) {
             net = net_map[bssid];
         } else {
             net = new wireless_network;
             net->bssid = bssid;
-            net_map[bssid] = net;
-            net_map_vec.push_back(net);
-            last_new_network = net;
+            net->tcpclient = this;
+            newnet = 1;
         }
 
         scanned = sscanf(in_data+hdrlen+18, "%d \001%255[^\001]\001 \001%255[^\001]\001 "
                          "%d %d %d %d %d %d %d %d %d %hd.%hd.%hd.%hd "
                          "%d %f %f %f %f %f %f %f %f %d %d %d %f %17s %d %d %d %d %d %d %d %f %f %f "
-                         "%lf %lf %lf %ld",
+                         "%lf %lf %lf %ld %ld"
+                         "%d %d %d",
                          (int *) &net->type, ssid, beacon,
                          &net->llc_packets, &net->data_packets, &net->crypt_packets, &net->interesting_packets,
                          &net->channel, &net->wep, (int *) &net->first_time, (int *) &net->last_time,
@@ -301,13 +308,21 @@ int TcpClient::ParseData(char *in_data) {
                          &net->best_quality, &net->best_signal, &net->best_noise,
                          &net->best_lat, &net->best_lon, &net->best_alt,
                          &net->aggregate_lat, &net->aggregate_lon, &net->aggregate_alt,
-                         &net->aggregate_points);
+                         &net->aggregate_points, &net->datasize,
+                         &net->turbocell_nid, (int *) &net->turbocell_mode, &net->turbocell_sat);
 
-        if (scanned < 44) {
+        if (scanned < 45) {
             // fprintf(stderr, "Flubbed network, discarding...\n");
+            delete net;
             return 0;
         }
 
+        if (newnet == 1) {
+            net_map[bssid] = net;
+            net_map_vec.push_back(net);
+            last_new_network = net;
+        }
+
         if (ssid[0] != '\002')
             net->ssid = ssid;
         if (beacon[0] != '\002')
@@ -318,21 +333,44 @@ int TcpClient::ParseData(char *in_data) {
 
         net->manuf_key = manuf_str;
         net->maxrate = maxrate;
+
     } else if (!strncmp(header, "*CLIENT", 64)) {
         short int ip[4];
 
+        mac_addr cmac;
         char cmac_str[18];
         char manuf_str[18];
 
         int scanned;
         float maxrate;
-        wireless_client *client = new wireless_client;
 
-        scanned = sscanf(in_data+hdrlen, "%17s %17s %d %d %d %17s %d %d %d %d %d "
+        // Find the bssid and mac so we can fill in our client or make a new one
+        if (sscanf(in_data+hdrlen, "%17s %17s", bssid_str, cmac_str) != 2)
+            return 0;
+
+        bssid = bssid_str;
+        wireless_client *client = NULL;
+        int nclient = 0;
+
+        map<mac_addr, wireless_network *>::iterator nmi = net_map.find(bssid);
+        if (nmi != net_map.end()) {
+            cmac = cmac_str;
+            map<mac_addr, wireless_client *>::iterator wci = nmi->second->client_map.find(cmac);
+            if (wci != nmi->second->client_map.end()) {
+                client = wci->second;
+            } else {
+                nclient = 1;
+                client = new wireless_client;
+            }
+        } else {
+            return 0;
+        }
+
+        scanned = sscanf(in_data+hdrlen+36, "%d %d %d %17s %d %d %d %d %d "
                          "%f %f %f %f %f %f %f %f %lf %lf "
                          "%lf %ld %f %d %d %d %d %d %d %d "
-                         "%f %f %f %d %hd.%hd.%hd.%hd",
-                         bssid_str, cmac_str, (int *) &client->type,
+                         "%f %f %f %d %hd.%hd.%hd.%hd %ld",
+                         (int *) &client->type,
                          (int *) &client->first_time, (int *) &client->last_time,
                          manuf_str, &client->manuf_score,
                          &client->data_packets, &client->crypt_packets,
@@ -346,22 +384,31 @@ int TcpClient::ParseData(char *in_data) {
                          &client->quality, &client->signal, &client->noise,
                          &client->best_quality, &client->best_signal, &client->best_noise,
                          &client->best_lat, &client->best_lon, &client->best_alt,
-                         (int *) &client->ipdata.atype, &ip[0], &ip[1], &ip[2], &ip[3]);
+                         (int *) &client->ipdata.atype, &ip[0], &ip[1], &ip[2], &ip[3],
+                         &client->datasize);
 
-        if (scanned < 38)
+        if (scanned < 37) {
+            if (nclient)
+                delete client;
             return 0;
+        }
 
         bssid = bssid_str;
-        client->mac = cmac_str;
+        client->mac = cmac;
         client->maxrate = maxrate;
 
         client->manuf_key = manuf_str;
 
+        client->tcpclient = this;
+
         for (unsigned int x = 0; x < 4; x++)
             client->ipdata.ip[x] = ip[x];
 
-        if (net_map.find(bssid) != net_map.end())
-            net_map[bssid]->client_map[client->mac] = client;
+        // Add it to the map, if its a new client.
+        if (nclient) {
+            net_map[bssid]->client_map[cmac] = client;
+            net_map[bssid]->client_vec.push_back(client);
+        }
 
     } else if (!strncmp(header, "*REMOVE", 64)) {
 
@@ -387,6 +434,13 @@ int TcpClient::ParseData(char *in_data) {
         memset(chan_details, 0, 1024);
         memset(chan_details_sec, 0, 1024);
 
+        old_num_networks = num_networks;
+        old_num_packets = num_packets;
+        old_num_crypt = num_crypt;
+        old_num_interesting = num_interesting;
+        old_num_noise = num_noise;
+        old_num_dropped = num_dropped;
+
         unsigned int numchan;
         if (sscanf(in_data+hdrlen, "%d %d %d %d %d %d %d %d %d %d %d%1023[^\n]\n",
                    &num_networks, &num_packets,
@@ -433,6 +487,11 @@ int TcpClient::ParseData(char *in_data) {
         if (sscanf(in_data+hdrlen, "%1023[^\n]\n", status) != 1)
             return 0;
         return CLIENT_NOTIFY;
+    } else if (!strncmp(header, "*ERROR", 64)) {
+        int discard;
+        if (sscanf(in_data+hdrlen, "%d %1023[^\n]\n", &discard, status) != 2)
+            return 0;
+        return CLIENT_NOTIFY;
     } else if (!strncmp(header, "*ALERT", 64)) {
         char alrmstr[2048];
         alert_info alrm;
@@ -447,9 +506,18 @@ int TcpClient::ParseData(char *in_data) {
         return CLIENT_ALERT;
     } else if (!strncmp(header, "*STRING", 64)) {
         char netstr[2048];
-        if (sscanf(in_data+hdrlen, "%2047[^\n]\n", netstr) != 1)
+        char bssid_str[18];
+        char source_str[18];
+        string_info strng;
+        if (sscanf(in_data+hdrlen, "%17s %17s %2047[^\n]\n", bssid_str, source_str, netstr) != 3)
             return 0;
-        strings.push_back(netstr);
+
+        strng.bssid = bssid_str;
+        strng.source = source_str;
+
+        strng.text = netstr;
+
+        strings.push_back(strng);
         if (strings.size() > maxstrings)
             strings.erase(strings.begin());
     } else if (!strncmp(header, "*PACKET", 64)) {
@@ -457,18 +525,20 @@ int TcpClient::ParseData(char *in_data) {
         memset(&packinfo, 0, sizeof(packet_info));
         char smac[18], dmac[18], bmac[18];
         short int sip[4], dip[4];
+        int sport, dport;
 
-        if (sscanf(in_data+hdrlen, "%d %d %d %d %d %17s %17s %17s "
+        if (sscanf(in_data+hdrlen, "%d %d %d %d %d %d %17s %17s %17s "
                    "\001%32[^\001]\001 %d %hd.%hd.%hd.%hd %hd.%hd.%hd.%hd %d %d %d "
                    "\001%16[^\001]\001\n",
                    (int *) &packinfo.type,
+                   (int *) &packinfo.subtype,
                    (int *) &packinfo.time,
                    &packinfo.encrypted, &packinfo.interesting, &packinfo.beacon,
                    smac, dmac, bmac,
                    packinfo.ssid,
                    (int *) &packinfo.proto.type,
                    &sip[0], &sip[1], &sip[2], &sip[3], &dip[0], &dip[1], &dip[2], &dip[3],
-                   (int *) &packinfo.proto.sport, (int *) &packinfo.proto.dport,
+                   (int *) &sport, (int *) &dport,
                    (int *) &packinfo.proto.nbtype, packinfo.proto.netbios_source) < 22)
             return 0;
 
@@ -476,6 +546,9 @@ int TcpClient::ParseData(char *in_data) {
         packinfo.dest_mac = dmac;
         packinfo.bssid_mac = bmac;
 
+        packinfo.proto.sport = sport;
+        packinfo.proto.dport = dport;
+
         for (unsigned int x = 0; x < 4; x++) {
             packinfo.proto.source_ip[x] = sip[x];
             packinfo.proto.dest_ip[x] = dip[x];
@@ -537,7 +610,7 @@ int TcpClient::FetchChannelPower(int in_channel) {
 
 int TcpClient::Send(const char *in_data) {
     if (!sv_valid) {
-        snprintf(errstr, 1024, "TcpClient poll() on an inactive connection.");
+        snprintf(errstr, 1024, "TcpClient send() on an inactive connection.");
         return -1;
     }
 
@@ -545,3 +618,29 @@ int TcpClient::Send(const char *in_data) {
 
     return 1;
 }
+
+void TcpClient::EnableProtocol(string in_protocol) {
+    char data[1024];
+
+    // We don't care about ACKS or even errors, so we just don't give an ID number
+    snprintf(data, 1024, "!0 ENABLE %s *\n", in_protocol.c_str());
+
+    Send(data);
+
+    protocol_map[in_protocol] = 1;
+
+}
+
+void TcpClient::RemoveProtocol(string in_protocol) {
+    char data[1024];
+
+    // We also don't care about acks or errors here
+    snprintf(data, 1024, "!0 REMOVE %s\n", in_protocol.c_str());
+
+    Send(data);
+
+    map<string, int>::iterator pritr = protocol_map.find(in_protocol);
+    if (pritr != protocol_map.end())
+        protocol_map.erase(pritr);
+}
+
