@@ -45,7 +45,7 @@ public:
 
     int CloseDump();
 
-    int DumpPacket(const packet_info *in_info, const pkthdr *in_header, const u_char *in_data);
+    int DumpPacket(const packet_info *in_info, const kis_packet *packet);
 
 protected:
     /* Stolen from libwiretap */
